/*
 * Copyright (C) 2015 Fondazione Istituto Italiano di Tecnologia
 *
 * Licensed under either the GNU Lesser General Public License v3.0 :
 * https://www.gnu.org/licenses/lgpl-3.0.html
 * or the GNU Lesser General Public License v2.1 :
 * https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * at your option.
 */

#include <iDynTree/Model/Indices.h>

namespace iDynTree
{

LinkIndex LINK_INVALID_INDEX  = -1;
std::string LINK_INVALID_NAME = "";

JointIndex JOINT_INVALID_INDEX = -1;
std::string JOINT_INVALID_NAME = "";

FrameIndex FRAME_INVALID_INDEX = -1;
std::string FRAME_INVALID_NAME = "";

DOFIndex   DOF_INVALID_INDEX   = -1;
std::string DOF_INVALID_NAME   = "";

TraversalIndex TRAVERSAL_INVALID_INDEX = -1;


}
